
"""
This is the Engage SDK for Django applications.

The SDK provides a command line utility to determine in advance whether your
Django application can be deployed on Engage and to package it up for
Engage. If you just want to get up and running quickly, take a look at the
Quickstart guide (the readme.rst file associated with this package). For
more detailed documentation, see the Engage Django SDK reference manual
on the genForma website.

This Engage SDK is Copyright 2010,2011 by genForma Corporation.

"""
