"""Version of the SDK
"""


# The following line should have VERSION, an equal sign, and the version number in single quotes.
# There should be no whitespace. This makes it easy to automatically extract the version number
VERSION='0.7.0'
